from dataclasses import dataclass, field

import openai
import os
import sys
import json
from pathlib import Path

@dataclass
class Cannoli:
    api_key: str= field(default=None)
    setup: dict = field(default=None)
    last_prompt: str = field(default='')
    response: dict = field(default=None)

    def __post_init__ (self):
        if not self.setup:
            self.load_default_settings()

        if not self.api_key:
            self.get_api_key()
        openai.api_key = self.api_key

    def load_default_settings(self):
        try:
            setup_filename = Path(__file__).resolve().parent / 'default_settings.json'
            with open(setup_filename, 'r', encoding='utf-8') as settings_file:
                self.setup = json.load(settings_file)
        except Exception:
            raise

    def get_api_key(self):
        self.api_key = os.getenv('OPENAI_API_KEY')
        if not self.api_key:
            print('Error: OPENAI_API_KEY is missing. Please, include it in your initiatlization script.')
            sys.exit()

    def __parse_response(self, response):
        if response.get('error'):
            print(f"API Error - response['error']['message']: {response['error']['message']}")
            return None
        generated_text = response.choices[0].text.strip()
        return generated_text


    def api_request(self, prompt=None):
        if prompt: 
            self.last_prompt = '' if not self.setup.get('prompt') else self.setup.get('prompt') 
            self.setup['prompt'] = prompt
        print(self.setup)
        self.response = openai.Completion.create(**self.setup)
        return self.response

    def quick_question(self, prompt):
        self.api_request(prompt)
        return self.__parse_response(self.response)


def main():
    # making quick questions
    demo = Cannoli()
    ans = demo.quick_question("what is a cannoli?")
    print(ans)

    # accessing the full response
    print(demo.response)

    # accessing the last prompt
    demo.quick_question('who is gandalf?')
    print(demo.last_prompt)


if __name__ == "__main__":
    main()


import pytest
from cannoli.cannoli import *

def test_check_default_values():
    default_values = {
        "engine": "text-davinci-003",
        "prompt": "Your are my personal assistant. Answer me the best as you can.",
        "max_tokens": 300,
        "temperature": 0.7
    }    
    cannoli = Cannoli()
    assert cannoli.setup == default_values  


def test_get_api_key():
    cannoli = Cannoli()
    assert len(cannoli.api_key)>0


def test_quick_question():
    cannoli = Cannoli()
    prompt = "answer me in one word, which country cannoli is?"
    assert cannoli.quick_question(prompt) == "Italy"


from setuptools import setup, find_packages

setup(
    name="cannoli",
    version="0.1.9",
    packages=find_packages(),
    include_packages_data=True,
    install_requires=[
        'openai',
        'pandas',
        'openpyxl'
    ],
    author="Narumi Abe, Bruna Luzzi",
    author_email="mail.narumi@gmail.com",
    description="Lib for prompt engineering using OpenAI",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url="https://github.com/naruminho/cannoli",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
